#version 150

#moj_import <light.glsl>

in vec3 Position;
in vec4 Color;
in vec2 UV0;
in ivec2 UV2;
in vec3 Normal;

uniform sampler2D Sampler0;
uniform sampler2D Sampler2;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform vec3 ChunkOffset;

uniform float GameTime;

out float vertexDistance;
out vec4 vertexColor;
out vec2 texCoord0;
out vec4 normal;
out float player_pos;

const float EPSYLON = 0.001;

const float TIMER = 1500.0;
const float SCALE = 8.0;
const float UP = 3.0;

vec2 hash(vec2 p) {
    return fract(sin(vec2(dot(p,vec2(127.1,311.7)),dot(p,vec2(269.5,183.3))))*43758.5453);
}

vec2 hash_2(vec2 i) {
	return hash(hash(i));
}

float cell_noise(vec2 par) {
	vec2 coord = par / 16.0 * SCALE;
	
	vec2 i_xz = floor(coord);
	vec2 xz = fract(coord);
	
	float min_val = 1.0;
	for (int x = -1; x < 2; x++) {
		for (int y = -1; y < 2; y++) {
			vec2 next = vec2(float(x),float(y));
			vec2 point = hash(i_xz + next);
			
			float dist = length(point + next - xz);
			min_val = min(min_val, dist);
		}
	}
	
	return min_val * UP;
}

vec3 upscale(vec3 in_pos, float add_x, float add_z, float r, float b) {
	float mult = r == 1.0 ? 8.0 : 0.0;
	vec3 out_pos = in_pos + vec3(add_x * 8.0, 0.0, add_z * mult);
	
	out_pos.x *= 4.0;
	out_pos.z *= 2.0 / r;
	
	out_pos.x -= 78.0;
	out_pos.z += b / 5.0 * 16.0;
	
	return out_pos;
}

void main() {
	vec3 f_pos = Position;
	float fog_offset = 1.0;
	
	vec3 c = texture(Sampler0, UV0).rgb;
	float c_sum = c.r + c.g + c.b;
	if (c_sum == 2.4) {
		float blue = c.b == 1.0 ? 5.0 : 0.0;
		
		float add_x = mod(f_pos.y, 4.0) - (c.g == 0.6 || c.g+c.b == 1.6 ? 4.0 : 0.0);
		float add_z = floor(f_pos.y / 4.0) - (c.r == 1.0 ? 2.0 : 0.0) - blue;
		
		f_pos.y = 16.0;
		vec3 pos = f_pos;
		
		float red = c.r == 1.0 ? 2.0 : 1.0;
		add_z /= red;
		pos.z /= red;
		
		vec2 time = vec2(GameTime * TIMER);
		pos.xz += time + vec2(add_x, add_z) * 16.0;
		
		float dy = cell_noise(pos.xz);
		pos.y += dy;
		f_pos.y += dy;
		
		float dx = (cell_noise(pos.xz + vec2(0.0, EPSYLON)) - cell_noise(pos.xz - vec2(0.0, EPSYLON))) / (2.0 * EPSYLON);
		float dz = (cell_noise(pos.xz + vec2(EPSYLON, 0.0)) - cell_noise(pos.xz - vec2(EPSYLON, 0.0))) / (2.0 * EPSYLON);
		
		normal = normalize(vec4(dz-dy, 1.0, dx-dy, 0.0));
		
		f_pos = upscale(f_pos, add_x, add_z, red, blue);
		fog_offset = ChunkOffset.z < 0.0 ? 2.0 : 1.15;
	} else {
		normal = ProjMat * ModelViewMat * vec4(Normal, 0.0);
	}
	
    gl_Position = ProjMat * ModelViewMat * vec4(f_pos + ChunkOffset, 1.0);
	
	vec3 fog = f_pos + ChunkOffset;
	fog.z *= fog_offset;
    vertexDistance = length((ModelViewMat * vec4(fog, 1.0)).xyz);
	
    vertexColor = Color * minecraft_sample_lightmap(Sampler2, UV2);
    texCoord0 = UV0;
	
	player_pos = ChunkOffset.z;
}
